<?php
/*
 * Copyright (c) 2013, MEDIASTUTTGART, http://mediastuttgart.de
 *
 * This file is released under the terms of the MIT license.
 * You can find the complete text in the attached LICENSE
 * file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class _M_Datatype_CKEditor extends _WV_Datatype_StringText {
	/**
	 * @param  sly_Form     $form
	 * @param  WV_IProperty $property
	 * @param  mixed        $value
	 * @return sly_Form
	 */
	public function renderForm(sly_Form $form, WV_IProperty $property, $value) {
		if ($value === false) {
			$default = $property->getDefault();
			$value   = $property->isMultilingual() ? array($default) : $default;
		}
		elseif (!$property->isMultilingual()) {
			$value = reset($value);
		}

		$name   = 'property_'.$property->getID().'_value';
		$params = $property->getParams();
		$editor = new sly_Form_CKEditor($name, $property->getTitle(), $value);
		$editor->setHelpText($property->getHelpText());

		extract($this->deserializeParams($params));

		if ($config) {
			$editor->setConfig($config);
		}

		if ($styleset) {
			$editor->setStyleset($styleset);
		}

		if ($usefullwidth) {
			$editor->useFullWidth();
		}

		if ($clicktoload) {
			$editor->clickToLoad(true);
		}

		$form->add($editor);

		return $form;
	}

	/**
	 * @return array
	 */
	protected function getDefaultParams() {
		return array(
			'minlength'    => 0,
			'maxlength'    => 65535,
			'config'       => null,
			'styleset'     => null,
			'usefullwidth' => false,
			'clicktoload'  => false
		);
	}
}
