<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class sly_Controller_Devutilsdocs extends sly_Controller_Backend implements sly_Controller_Interface {
	public function indexAction() {
		$request = $this->getRequest();
		$curDoc  = $request->get('doc', 'string', '');

		if (!empty($curDoc)) {
			return $this->documentView($curDoc);
		}

		$articleTypes = $this->collectTypes();
		$modules      = $this->collectModules();
		$extended     = $request->get('extended', 'boolean', false);

		$this->pageHeader();
		$this->render('articletypes.phtml', compact('articleTypes', 'modules', 'extended'), false);
	}

	protected function documentView($doc) {
		$container  = $this->getContainer();
		$docService = $container['wv-documentation-service'];
		$docs       = $docService->getDocuments(false);

		if (!isset($docs[$doc])) {
			return $this->redirectResponse();
		}

		$title = $docService->getDocumentTitle($doc);
		$html  = $docService->getParsedDocument($doc, true);

		$this->pageHeader();
		$this->render('doc.phtml', compact('html', 'title'), false);
	}

	protected function pageHeader() {
		$layout = $this->getContainer()->getLayout();
		$layout->pageHeader('Dokumentation');
	}

	protected function getViewFolder() {
		return DEVUTILS_PATH.'views/';
	}

	protected function collectTypes() {
		$container	 = $this->getContainer();
		$types		 = array();
		$typeService = $container->getArticleTypeService();
		$tplService	 = $container->getTemplateService();
		$allModules	 = count($this->collectModules());

		foreach ($typeService->getArticleTypes() as $typeName => $title) {
			$typeData = array(
				'title'		  => $title,
				'description' => $typeService->get($typeName, 'description', t('devutils_documentation_no_description'))
			);

			//////////////////////////////////////////////////////////
			// which slots are available

			$template  = $typeService->getTemplate($typeName);

			if (!empty($template)) {
				$slotNames = $tplService->getSlots($template);
				$slots	   = array();

				foreach ($slotNames as $slotName) {
					$modules = $typeService->getModules($typeName, $slotName);

					if (count($modules) === $allModules) {
						$modules = array('<alle>');
					}

					$slots[$slotName] = array(
						'title'	  => $tplService->getSlotTitle($template, $slotName),
						'modules' => $modules
					);
				}

				$typeData['slots'] = $slots;
			}
			//////////////////////////////////////////////////////////
			// put the data in the result array

			$types[$typeName] = $typeData;
		}

		uasort($types, array($this, 'sortHelper'));
		return $types;
	}

	public function sortHelper($elA, $elB) {
		return strnatcasecmp($elA['title'], $elB['title']);
	}

	protected function collectModules() {
		$container = $this->getContainer();
		$modules   = array();
		$service   = $container->getModuleService();

		foreach ($service->getModules() as $name => $title) {
			$modules[$name] = array(
				'title'		  => $title,
				'description' => $service->get($name, 'description', t('devutils_documentation_no_description'))
			);
		}

		uasort($modules, array($this, 'sortHelper'));
		return $modules;
	}

	public function checkPermission($action) {
		return $this->getCurrentUser() !== null;
	}
}
