<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

class WV8_Form extends sly_Form {
	protected $provider;
	protected $settings;
	protected $layout;
	protected $with_hidden = false;

	private $hasCustomBar;

	/**
	 * Constructor
	 */
	public function __construct(WV8_Settings_Provider $provider, sly_Router_Backend $router, sly_Layout $layout) {
		$url = $router->getAbsoluteUrl('globalsettings_settings', 'save', null, '&');
		parent::__construct($url, 'POST', '');

		$this->provider     = $provider;
		$this->settings     = array();
		$this->layout       = $layout;
		$this->hasCustomBar = false;

		$this->addClass('wv8-form');
		$this->setSubmitButton(null);
		$this->setResetButton(null);
	}

	public function addPage($page) {
		$groups = $this->provider->getGroups(null, $page, $this->with_hidden);
		$sorted = WV8_Util::translateAndSortList($groups);

		foreach ($sorted as $idx => $group) {
			$this->addGroup($page, $groups[$idx]);
		}

		return $this;
	}

	public function addGroup($page, $group) {
		$groupID   = WV8_Util::getGroupID($page, $group);
		$groupName = WV8_Util::translate($group);
		$settings  = $this->provider->getSettings(null, $page, $group);

		$fieldset = $this->beginFieldset($groupName);
		$fieldset->setAttribute('data-wv8id', $groupID);

		foreach ($settings as $setting) {
			if ($setting->getData('hidden') && !$this->with_hidden) continue;
			$setting->renderAsFormElement($this);
			$this->settings[] = $setting->getID();
		}

		return $this;
	}

	public function getSettingIDs() {
		return $this->settings;
	}

	/**
	 * Render the form
	 *
	 * Renders the form and returns its content.
	 *
	 * @param  boolean     $omitFormTag  set this to true if you want to use your own <form> tag
	 * @param  sly_Request $request      the request to use or null for the global one
	 * @return string                    the generated XHTML
	 */
	public function render($omitFormTag = false, sly_Request $request = null) {
		if (!$this->hasCustomBar) {
			$submit = new sly_Form_Input_Button('submit', 'submit', t('save'));
			$reset  = new sly_Form_Input_Button('reset', 'reset', t('reset'));
			$bar    = new sly_Form_ButtonBar(compact('submit', 'reset'));

			$submit->addClass('sly-form-submit');
			$bar->addFormRowClass('wv8-dont-hide');
			$this->add($bar);

			$this->hasCustomBar = true;
		}

		$this->addHiddenValue('wv8settings', implode(',', $this->settings));

		$devUtils = 'webvariants/developer-utils';
		$wv8      = 'webvariants/global-settings';

		$this->layout->addJavaScriptFile(Util::addOnUri($devUtils, 'js/jquery.wvtogglefieldset.min.js'));
		$this->layout->addJavaScriptFile(Util::addOnUri($devUtils, 'js/jquery.cookie.min.js'));
		$this->layout->addJavaScriptFile(Util::addOnUri($wv8, 'js/globalsettings.js'));
		$this->layout->addCSSFile(Util::addOnUri($wv8, 'css/globalsettings.less'));

		return parent::render($omitFormTag, $request);
	}

	/**
	 * Change the action (url) of the form
	 *
	 * @param string $action
	 */
	public function setAction($action) {
		$this->action = $action;
	}

	/**
	 * Enable/ disable to show even hidden settings
	 *
	 * @param boolean $with_hidden
	 */
	public function setWithHidden($with_hidden = true) {
		$this->with_hidden = $with_hidden;
	}
}
