<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use wv\metainfo\Metainfo;
use wv\metainfo\Property\Article as ArticleProperty;

class sly_Controller_Devutilsdocs_Metainfo extends sly_Controller_Devutilsdocs {
	protected function getViewFolder() {
		return _WV2_PATH.'templates/';
	}

	public function indexAction() {
		$this->init();
		$metainfos = $this->collectMetainfos();
		$this->render('documentation.phtml', compact('metainfos'), false);
	}

	protected function collectMetainfos() {
		$metainfos   = array();
		$container   = $this->getContainer();
		$typeService = $container->getArticleTypeService();
		$propService = $container['metainfo-service-property'];
		$allTypes    = count($typeService->getArticleTypes());
		$types       = array(
			Metainfo::TYPE_ARTICLE  => t('articles'),
			Metainfo::TYPE_CATEGORY => t('categories'),
			Metainfo::TYPE_MEDIUM   => t('media'),
			Metainfo::TYPE_USER     => t('users')
		);

		foreach ($types as $type => $name) {
			$list             = $propService->findPropertiesByType($type);
			$metainfos[$type] = array('title' => $name, 'infos' => array());

			foreach ($list as $metainfoName => $metainfo) {
				$description   = $metainfo->getData('description', $metainfo->getHelpText());
				$isArticleInfo = $metainfo instanceof ArticleProperty;

				if (empty($description)) {
					$description = '<'.t('metainfo_no_description').'>';
				}

				if ($isArticleInfo) {
					$atypes = $metainfo->getArticleTypes();
				}
				else {
					$atypes = null;
				}

				if (count($atypes) === $allTypes) {
					$atypes = array('<'.t('all').'>');
				}
				elseif ($atypes) {
					foreach ($atypes as $idx => $typeName) {
						$atypes[$idx] = sly_translate($typeService->getTitle($typeName));
					}
				}

				$metainfos[$type]['infos'][$metainfoName] = array(
					'title'       => $metainfo->getTitle(),
					'description' => $description,
					'types'       => $atypes,
					'metapage'    => $isArticleInfo ? $metainfo->getMetapage() : false
				);
			}

			if (empty($metainfos[$type]['infos'])) {
				unset($metainfos[$type]);
			}
			else {
				uasort($metainfos[$type]['infos'], array($this, 'sortHelper'));
			}
		}

		return $metainfos;
	}

	public function sortHelper($elA, $elB) {
		return strnatcasecmp($elA['title'], $elB['title']);
	}

	public function checkPermission($action) {
		return $this->getCurrentUser() !== null;
	}
}
