<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Property;

use wv\metainfo\Metainfo;

class Article extends Property {
	protected $metapage;      ///< boolean  wenn true, wird die Information nur auf der separaten Meta-Seite von Artikeln angezeigt
	protected $articleTypes;  ///< array    Liste von Artikeltypen, denen diese Information zugewiesen ist

	public function __construct($name, array $data) {
		parent::__construct(Metainfo::TYPE_ARTICLE, $name, $data);

		$this->multilingual = false;
		$this->metapage     = (bool) $this->getData('metapage', false);
		$this->articleTypes = $this->getData('types', array());

		if (empty($this->articleTypes)) {
			$this->articleTypes = Metainfo::getContainer()->getArticleTypeService()->getArticleTypes(true);
		}
	}

	public function renderForm(\sly_Form $form, array $values) {
		$form->add(new \sly_Form_Fragment('<div id="wv2_metainfo'.$this->getID().'_row" class="wv2_metainfo_row">'));
		$result = parent::renderForm($form, $values);
		$form->add(new \sly_Form_Fragment('</div>'));

		return $result;
	}

	public function getMetapage() {
		return $this->metapage;
	}

	public function getArticleTypes() {
		return $this->articleTypes;
	}
}
