<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

abstract class WV42_Renderer_SimpleTokenList extends WV42_Renderer_Base {	
	/**
	 * render form
	 *
	 * @param  sly_Form        $form
	 * @param  pac_Role        $role
	 * @param  pac_Destination $destination
	 * @param  array           $destConfig
	 * @return sly_Form
	 */
	public function render(sly_Form $form, pac_Role $role, pac_Destination $destination, array $destConfig) {
		$tokens = $destConfig['token'];

		if (count($tokens) !== 1) {
			throw new LogicException('The SimpleTokenList renderer is only suitable for destinations with one token.');
		}

		// make sure we don't have to call our database a billion times
		$this->buildCache($role, $destination);

		// prepare token
		$destID = $destination->getId();
		$token  = key($tokens);
		$title  = sly_translate(reset($tokens), false);

		// fetch values
		$values = $this->getValues();
		$granted = array();

		foreach (array_keys($values) as $objectID) {
			if (isset($this->dataCache[$objectID]) && in_array($token, $this->dataCache[$objectID])) {
				$granted[] = $objectID;
			}
		}

		// build list
		$list = new WV42_Form_SimpleTokenList($destID, $token, $title, $granted, $values);

		// add element
		$form->beginFieldset(sly_translate($destConfig['title'], false), 'destination_'.$destID);
		$form->add($list);

		return $form;
	}
}
