<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

define('_WV24_PATH', __DIR__.DIRECTORY_SEPARATOR);

use wv\realurl2\Listener;

if (sly_Core::isBackend()) {
	$container['sly-i18n']->appendFile(_WV24_PATH.'i18n');
}

// init container

$container['realurl2-service'] = $container->share(function($container) {
	return new wv\realurl2\Service($container);
});

$container['realurl2-storage'] = $container->share(function($container) {
	return new wv\realurl2\Storage($container['sly-persistence'], $container['sly-cache']);
});

$container['realurl2-resolver'] = $container->share(function($container) {
	return new wv\realurl2\Resolver($container['realurl2-storage'], $container['sly-cache'], $container['sly-persistence']);
});

$container['realurl2-dispatcher'] = $container->share(function($container) {
	return new wv\realurl2\Dispatcher($container['realurl2-resolver'], sly_Core::isDeveloperMode(), \wv\realurl2\Util::getDefaultFragments());
});

$container['realurl2-cache-runtime'] = /* do not share this */ function($container) {
	return new wv\realurl2\Cache\Runtime($container['sly-persistence'], $container['sly-service-article']);
};

$container['realurl2-listener-articles'] = $container->share(function($container) {
	return new Listener\Articles($container);
});

$container['realurl2-listener-categories'] = $container->share(function($container) {
	return new Listener\Categories($container);
});

$container['realurl2-listener-languages'] = $container->share(function($container) {
	return new Listener\Languages($container);
});

$container['realurl2-listener-system'] = $container->share(function($container) {
	return new Listener\System($container);
});

// event listeners

$dispatcher = $container['sly-dispatcher'];
$listeners  = array(
	'%realurl2-listener-articles%' => array(
		'SLY_ART_ADDED'              => 'artAdded',
		'SLY_ART_COPIED'             => 'artAdded',
		'SLY_ART_UPDATED'            => 'artUpdated',
		'SLY_ART_DELETED'            => 'artDeleted',
		'SLY_ART_ONLINE'             => 'artOnline',
		'SLY_ART_OFFLINE'            => 'artOffline',
		'SLY_ART_META_UPDATED'       => 'artMetaUpdated',
		'SLY_ART_TO_STARTPAGE'       => 'artToStartpage',
		'SLY_ART_MOVED'              => 'artMoved',
		'SLY_ART_TOUCHED'            => 'artTouched',
		'SLY_ART_META_FORM_FIELDSET' => 'artMetaFormSection'
	),

	'%realurl2-listener-categories%' => array(
		'SLY_CAT_ADDED'     => 'catAdded',
		'SLY_CAT_UPDATED'   => 'catUpdated',
//		'SLY_CAT_DELETED'   => 'catDeleted',     // no need, SLY_ART_DELETED takes care of everything
		'SLY_CAT_FORM_EDIT' => 'catFormEdit',
		'SLY_CAT_MOVED'     => 'catMoved'
	),

	'%realurl2-listener-languages%' => array(
		'SLY_CLANG_ADDED'   => 'clangAdded',
		'SLY_CLANG_DELETED' => 'clangDeleted'
	),

	'%realurl2-listener-system%' => array(
		// react when language fragments and base URL change
		'WV8_SETTINGS_UPDATED' => 'settingsUpdated',

		// react when the system config has been changed
		'SLY_SETTINGS_UPDATED' => 'slySettingsUpdated',

		// rewrite URLs
		'URL_REWRITE' => 'urlRewrite',

		// resolve requests
		'SLY_RESOLVE_ARTICLE' => 'resolveArticle',

		// replace 'realurl2://' URLs
		'SLY_ARTICLE_OUTPUT' => 'outputFilter',

		// cache clearing stuff
		'SLY_SYSTEM_CACHES' => 'systemCacheList',
		'SLY_CACHE_CLEARED' => 'cacheCleared',

		// model extensions
		'SLY_MODEL_ARTICLE_ISSKIPPED'   => 'articleIsSkipped',
		'SLY_MODEL_ARTICLE_GETFRAGMENT' => 'articleGetFragment'
	)
);

foreach ($listeners as $service => $events) {
	foreach ($events as $event => $listener) {
		$dispatcher->addListener($event, array($service, $listener));
	}
}
