<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Listener;

use sly_Model_Language;
use wv\realurl2\Util;
use wv\realurl2\Generator\Article as ArticleGenerator;

/**
 * Sally-Eventlistener
 *
 * Diese Klasse sammelt zu einem Großteil nur Listener, um den URL-Cache immer
 * aktuell zu halten. Für gewöhnlich will niemand diese Methoden direkt
 * aufrufen.
 *
 * @author Christoph
 * @since  forever
 */
class Languages extends Base {
	/**
	 * SLY_CLANG_ADDED: Sprache wurde hinzugefügt
	 */
	public function clangAdded(sly_Model_Language $language) {
		$locale = $language->getLocale();
		$id     = $language->getId();

		if (empty($locale)) {
			$locale = 'lng'.$id;
		}
		else {
			$locale = strtolower(str_replace('_', '-', $locale));
		}

		// set the new default fragment for this language

		$request  = $this->container['sly-request'];
		$provider = $this->container['wv8-provider'];
		$base     = mb_substr($request->getBaseUrl(true, 'http'), 5).'/'.$locale;
		$frgmt    = $provider->getSetting('realurl2', 'default_fragment');
		$lower    = $provider->getSetting('realurl2', 'lowercase');
		$clean    = $provider->getSetting('realurl2', 'cleanup');

		$frgmtValues = $frgmt->getValues(true);
		$lowerValues = $lower->getValues(true);
		$cleanValues = $clean->getValues(true);

		$frgmtValues[$id] = $base;
		$lowerValues[$id] = 1;
		$cleanValues[$id] = 1;

		$service = $this->container['wv8-manager'];
		$service->storeValues($frgmt, $frgmtValues);
		$service->storeValues($lower, $lowerValues);
		$service->storeValues($clean, $cleanValues);

		// generate fresh URLs

		if (Util::hasSiteStartArticle()) {
			// prepare
			$cache     = $this->getRuntimeCache();
			$storage   = $this->getStorage();
			$generator = $this->getService()->getGenerator($cache);

			// prewarm
			$cache->prewarmAll($id);

			// generate & store
			$articles = Util::getTree(null);
			$this->rebuildList($articles, $id, $generator, $storage);
		}
	}

	/**
	 * CLANG_DELETED: Sprache wurde gelöscht
	 */
	public function clangDeleted(sly_Model_Language $language) {
		if (!Util::hasSiteStartArticle()) return;
		$this->getStorage()->clearByClang($language->getId());
	}
}
