<?php
/*
 * Copyright (c) 2014, webvariants GmbH & Co. KG, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class sly_Controller_Mediapool_Sync extends sly_Controller_Mediapool {
	public function indexAction() {
		$this->init();

		$diff = $this->getFileDiff();

		if (empty($diff)) {
			$flash = $this->getContainer()->getFlashMessage();
			$flash->appendInfo(t('no_file_diffs_found'));

			print sly_Helper_Message::renderFlashMessage($flash);
		}
		else {
			$this->render('mediapool/sync.phtml', array('diffFiles' => $diff), false);
		}
	}

	public function syncAction() {
		$request  = $this->getRequest();
		$selected = $request->postArray('sync_files', 'string');
		$flash    = $this->getFlashMessage();

		if (!empty($selected)) {
			$title = $request->post('ftitle', 'string', '');
			$diff  = $this->getFileDiff();
			$cat   = $this->getCurrentCategory();
			$count = 0;

			foreach ($selected as $hash) {
				if (isset($diff[$hash]) && $this->syncMedium($diff[$hash], $cat, $title)) {
					++$count;
				}
			}

			$flash->appendInfo(t('files_synced', $count));
			return $this->redirectResponse();
		}

		$flash->appendWarning(t('no_files_selected'));
		return $this->indexAction();
	}

	public function checkPermission($action) {
		if (!parent::checkPermission($action)) return false;

		if ($action === 'sync') {
			sly_Util_Csrf::checkToken();
		}

		return true;
	}

	protected function syncMedium($filename, $category, $title) {
		$fixedName = sly_Util_Directory::fixWindowsDisplayFilename($filename);
		$container = $this->getContainer();
		$service   = $container->getMediumService();
		$fs        = $container->get('sly-filesystem-media');

		if ($fixedName !== $filename) {
			// move file to cleaned filename
			$fs->rename($filename, $fixedName);
		}

		if ($title === '') {
			$title = sly_Util_File::stripExtension($fixedName);
			$title = str_replace('_', ' ', $title);
			$title = ucfirst($title);
		}

		// add file to database

		try {
			$service->add($fixedName, $title, $category);
			return true;
		}
		catch (sly_Exception $e) {
			return false;
		}
	}

	protected function getFilesFromFilesystem() {
		$fs   = $this->getContainer()->get('sly-filesystem-media');
		$list = $fs->listKeys();

		return $list['keys'];
	}

	protected function getFilesFromDatabase() {
		$db    = $this->getContainer()->getPersistence();
		$files = array();

		$db->select('file', 'filename');
		foreach ($db->all() as $row) $files[] = $row['filename'];

		return $files;
	}

	protected function getFileDiff() {
		$database   = $this->getFilesFromDatabase();
		$filesystem = $this->getFilesFromFilesystem();
		$diff       = array_diff($filesystem, $database);
		$res        = array();

		// Do not use the filename as the array's key to avoid problems
		// when the filename contains broken characters.

		foreach ($diff as $filename) {
			$hash = substr(md5($filename), 0, 12);
			$res[$hash] = $filename;
		}

		return $res;
	}
}
